<?php 
// $Id: node_import.api.inc,v 1.7 2006/09/22 07:24:23 robrechtj Exp $

/**
 * @file
 * Public API of node_import to import CSV files with nodes.
 */

ini_set('auto_detect_line_endings', TRUE);

/**
 * Loads the hooks for the supported modules.
 */
function node_import_load_supported() {
  static $loaded = FALSE;
  if (!$loaded) {
    $path = drupal_get_path('module', 'node_import') . '/supported';
    $files = system_listing('.*\.inc$', $path, 'name', 0);
    foreach ($files as $module_name => $file) {
      if (module_exist($module_name)) {
        include_once($file->filename);
      }
    }
    $loaded = TRUE;
  }
}
node_import_load_supported();

/**
 * Get a list of supported nodes types.
 *
 * @param $check_permission
 *   Boolean. If TRUE, only returns the types for which the
 *   user has 'create' permissions. If FALSE, then all types
 *   which node_import supports are returned (even if the user
 *   is not allowed to create the nodes).
 *
 * @return
 *   Array ($type => $name) with all supported types.
 *
 * @see hook_node_import_types()
 */
function node_import_types($check_permission = TRUE) {
  static $types = NULL;
  static $permitted_types = NULL;

  if (!isset($types)) {
    $types = array();
    foreach ((array) module_invoke_all('node_import_types') as $type => $name) {
      $types[$type] = $name;
    }
    asort($types);
  }

  if (!isset($permitted_types)) {
    $permitted_types = array();
    foreach ($types as $type => $name) {
      if (node_access('create', $type)) {
        $permitted_types[$type] = $name;
      }
    }
  }

  return $check_permission ? $permitted_types : $types;
}

/**
 * Get a list of supported node fields for a given type.
 *
 * @param $type
 *   String. Node type (key in the array returned by node_import_types()).
 *
 * @return
 *   Array ($field => $name) with all the supported fields
 *   for this node type.
 *
 * @see hook_node_import_fields()
 */
function node_import_fields($type) {
  static $fields = array();
  if (!isset($fields[$type])) {
    $fields[$type] = (array)module_invoke_all('node_import_fields', $type);
  }
  return $fields[$type];
}

/**
 * Try to find a likely mapping for given headers.
 *
 * @param $type
 *   String. Node type (key in the array returned by node_import_types()).
 * @param $headers
 *   Array of string. Column headers of a CSV or TSV file.
 *
 * @return
 *   Array of string. A likely mapping given the $type and $header.
 */
function node_import_automap($type, $headers) {
  $sql = "SELECT mapping FROM {node_import_mappings} WHERE type = '%s' AND csv_headers = '%s'";
  $obj = db_fetch_object(db_query($sql, $type, serialize($headers)));
  if ($obj) {
    return unserialize($obj->mapping);
  }
}
