<?php
// $Id: location.inc,v 1.3 2006/09/17 10:57:53 robrechtj Exp $

/**
 * Implementation of hook_node_import_fields().
 */
function location_node_import_fields($type) {
  if (!variable_get('location_'. $type, FALSE)) {
    return;
  }

  $location_fields = array(
    'name' => t('Location: Name'),
    'street' => t('Location: Street'),
    'city' => t('Location: City'),
    'province' => t('Location: State or province'),
    'postal_code' => t('Location: Postal code'),
    'country' => t('Location: Country'),
  );

  $fields = array();
  foreach ($location_fields as $field => $title) {
    if (variable_get('location_'. $field .'_'. $type, 0)) {
      $fields['node_import_location_'. $field] = $title;
    }
  }
  if (in_array('node_import_location_street', array_keys($fields))) {
    $fields['node_import_location_additional'] = t('Location: Additional address line');
  }
  $fields['node_import_location_latitude'] = t('Location: Latitude');
  $fields['node_import_location_longitude'] = t('Location: Longitude');
  return $fields;
}

/**
 * Implementation of hook_node_import_prepare().
 */
function location_node_import_prepare(&$node, $preview = FALSE) {
  if (!variable_get('location_'. $node->type, FALSE)) {
    return;
  }

  $errors = array();
  $location = array();

  $location_fields = array(
    'name', 'street', 'additional',
    'city', 'province', 'postal_code', 'country',
    'latitude', 'longitude',
  );

  foreach ($location_fields as $field) {
    $importfield = 'node_import_location_'. $field;
    if (isset($node->$importfield)) {
      $location[$field] = trim($node->$importfield);
      unset($node->$importfield);
    }
  }

  // Try to find a valid country and province:
  $country_code = '';
  if (isset($location['country']) && $location['country'] != '') {
    $country_code = _node_import_location_get_country_code($location['country']);
    if ($country_code == '') {
      $errors[] = t('%input is not a valid country name or code.', array('%input' => theme('placeholder', $location['country'])));
    }
    else {
      $location['country'] = $country_code;
    }
  }

  $province_code = '';
  if (isset($location['province']) && $location['province'] != '') {
    $province_code = _node_import_location_get_province_code($location['province'], $country_code);
    if ($province_code == '') {
      $errors[] = t('%input is not a valid state or province name or code.', array('%input' => theme('placeholder', $location['province'])));
    }
    else {
      $location['province'] = $province_code;
      $country_code = substr($province_code, 0, 2);
    }
  }

  if (!isset($location['country']) || $location['country'] == '') {
    if ($country_code != '') {
      $location['country'] = $country_code;
    }
    else {
      $location['country'] = variable_get('location_default_country', 'us');
    }
  }

  //TODO: check required fields?
  
  // Add geocoding if available and needed.
  $geocode_fn = '';
  if ($location['country']) {
    $geocode_fn = 'location_latlon_exact_'. strtolower(trim($location['country']));
  }
  if (!$preview
      && $geocode_fn != ''
      && function_exists($geocode_fn)
      && (!$location['latitude'] || !$location['longitude'])) {
    $latlon = $geocode_fn($location);
    $location['latitude'] = $latlon['lat'];
    $location['longitude'] = $latlon['lon'];
  }
  if (isset($location['latitude']) && isset($location['longitude'])) {
    $location['source'] = LOCATION_LATLON_USER_SUBMITTED;
  }

  if (!empty($location)) {
    $node->location = $location;
  }

  return $errors;
}

/**
 * Convert the country from human-readable form
 * to two-letter country code.
 */
function _node_import_location_get_country_code($country) {
  static $iso_list;
  static $country_list;

  if (!isset($iso_list)) {
    $iso_list = location_get_iso3166_list();
    $country_list = array_flip(array_map('strtolower', $iso_list));
  }

  $country = trim(strtolower($country));

  if (isset($iso_list[$country])) {
    return $country;
  }

  if (isset($country_list[$country])) {
    return $country_list[$country];
  }

  return '';
}

/**
 * Convert the province from human-readable form
 * to a province code.
 */
function _node_import_location_get_province_code($province, $country) {
  $form = _location_province_select_options('', FALSE, $country);
  $codes = $form['#options'];

  $province = strtolower($province);

  foreach ($codes as $code => $name) {
    if (in_array(strtolower($code), array($province, "$country-$province")) || $province == strtolower($name)) {
      return $code;
    }
  }

  return '';
}
